/******************************************************************************
* File Name:   main.c
*
* Description:
* This is the source code for the PSoC 6 MCU: Bluetooth Low Energy Firmware Upgrade (DFU).
* Example for ModusToolbox.
* This file implements the DFUSDK based Bootloader.
*
* Related Document: See README.md.
*
*******************************************************************************
* $ Copyright 2021 Cypress Semiconductor $
*******************************************************************************/

#include "cy_pdl.h"
#include "cyhal.h"
#include "cybsp.h"
#include "cycfg_pins.h"
#include "debug.h"
#include "cy_dfu.h"
#include "dfu_user.h"
#include "cycfg_ble.h"
#include "cyhal_gpio.h"
#include "cy_retarget_io.h"
#include "transport_ble.h"

/*******************************************************************************
* Macros
********************************************************************************/

/* Interval for LED toggle: 1  blink per second. */
#define    LED_TOGGLE_INTERVAL_MS                  (500u)

/* User button interrupt priority. */
#define GPIO_INTERRUPT_PRIORITY                    (7u)

/* Timeout for Cy_DFU_Continue(), in milliseconds. */
#define DFU_SESSION_TIMEOUT_MS                     (20u)

/* Application ID */
#define USER_APP_ID                                (1u)

/* DFU idle wait timeout: 300 seconds.*/
#define DFU_IDLE_TIMEOUT_MS                        (300000u)

/* DFU session timeout: 5 seconds. */
#define DFU_COMMAND_TIMEOUT_MS                     (5000u)

/*******************************************************************************
* Function Prototypes
********************************************************************************/
static void user_button_event_handler(void *handler_arg, cyhal_gpio_event_t event);
static cy_en_dfu_status_t handle_metadata(cy_stc_dfu_params_t *params);
static cy_en_dfu_status_t copy_row(uint32_t dest, uint32_t src, uint32_t rowSize, cy_stc_dfu_params_t * params);
static cy_en_dfu_status_t restart_dfu(uint32_t *state, cy_stc_dfu_params_t *dfu_params );
static cy_en_dfu_status_t manage_bootable_app(cy_stc_dfu_params_t *dfu_params);
static uint32_t counter_timeout_seconds(uint32_t seconds, uint32_t timeout);
void Cy_OnResetUser(void);

/*******************************************************************************
* Global Variables
********************************************************************************/
volatile static bool is_user_event_detected = false;

/* Interrupt callback data */
cyhal_gpio_callback_data_t cb_data =
{
    .callback     = user_button_event_handler,
    .callback_arg = NULL
};



/*******************************************************************************
* Bluetooth low energy DFU specific
********************************************************************************/

/* BLE GAPP Connection Settings. */
#define CYBLE_GAPP_CONNECTION_INTERVAL_MIN  (0x000Cu) /* 15 ms - (N * 1,25)*/
#define CYBLE_GAPP_CONNECTION_INTERVAL_MAX  (0x000Cu) /* 15 ms */
#define CYBLE_GAPP_CONNECTION_SLAVE_LATENCY (0x0000u)
#define CYBLE_GAPP_CONNECTION_TIME_OUT      (0x00C8u) /* 2000 ms */

uint8 alert_level = CY_BLE_NO_ALERT;

void AppCallBack(uint32 event, void* eventParam);

void ble_ias_callback(uint32 event, void *eventParam);
void ble_findme_status(void);

struct image_version {
    uint8_t iv_major;
    uint8_t iv_minor;
    uint16_t iv_revision;
    uint32_t iv_build_num;
};

/** Image header.  All fields are in little endian byte order. */
struct image_header {
    uint32_t ih_magic;
    uint32_t ih_load_addr;
    uint16_t ih_hdr_size;           /* Size of image header (bytes). */
    uint16_t ih_protect_tlv_size;   /* Size of protected TLV area (bytes). */
    uint32_t ih_img_size;           /* Does not include header. */
    uint32_t ih_flags;              /* IMAGE_F_[...]. */
    struct image_version ih_ver;
    uint32_t _pad1;
};



/*******************************************************************************
* Function Name: main
********************************************************************************
* Summary:
* This is the bootloder main function for CM4 CPU.
*  1. If application started from Non-Software reset it validates app #1
*		--> If app#1 is valid it switches to app#1, else goto #2.
*  2. Start DFU communication.
*  3. If updated application has been received it validates this app.
*  4. If app#1 is valid it switches to it, else wait for new application.
*  5. If DFU_IDLE_TIMEOUT_MS seconds has passed and no new application has been received
*     then validate app#1, if it is valid then switch to it, else continue as is.
*     #5 will be repeated for every DFU_IDLE_TIMEOUT_MS seconds, if the new firmware is not received.
*
* Parameters:
*  void
*
* Return:
*  int
*
*******************************************************************************/
int main(void)
{
    cy_rslt_t result = CY_RSLT_SUCCESS;

    result = cybsp_init() ;
    CY_ASSERT(result == CY_RSLT_SUCCESS);

    /* Used to count seconds. */
    uint32_t count = 0;

    /* Status codes for DFU API. */
    cy_en_dfu_status_t status;

    /* Timeout variable. */
    uint32_t timeout_seconds = 0;

    /*
    * DFU state, one of the:
    * - CY_DFU_STATE_NONE
    * - CY_DFU_STATE_UPDATING
    * - CY_DFU_STATE_FINISHED
    * - CY_DFU_STATE_FAILED
    */
    uint32_t state = CY_DFU_STATE_NONE;

    /* Buffer to store DFU commands. */
    CY_ALIGN(4) static uint8_t buffer[CY_DFU_SIZEOF_DATA_BUFFER];

    /* Buffer for DFU data packets for transport API. */
    CY_ALIGN(4) static uint8_t packet[CY_DFU_SIZEOF_CMD_BUFFER];

    /* DFU params, used to configure DFU. */
    cy_stc_dfu_params_t dfu_params;

    /* Initialize dfu_params structure. */
    dfu_params.timeout          = DFU_SESSION_TIMEOUT_MS;
    dfu_params.dataBuffer       = &buffer[0];
    dfu_params.packetBuffer     = &packet[0];

    status = Cy_DFU_Init(&state, &dfu_params);
    CY_ASSERT(CY_DFU_SUCCESS == status);

    /* Ensure DFU Metadata is valid. */
    status = handle_metadata(&dfu_params);
    CY_ASSERT(CY_DFU_SUCCESS == status);

    /* In the case of non-software reset check if there is a valid app image.
     * If these is - switch to it.
     */
    if( (Cy_SysLib_GetResetReason() != CY_SYSLIB_RESET_SOFT) )
    {
        /* Boot to application image, if valid. */
        status = manage_bootable_app(&dfu_params);

        if (status == CY_DFU_SUCCESS)
        {
           /* Never returns from here. */
           Cy_DFU_ExecuteApp(USER_APP_ID);
        }

        /* Continue DFU execution (below), if user application not valid !. */
    }

    /* Initialize retarget-io to use the debug UART port */
	result = cy_retarget_io_init(CYBSP_DEBUG_UART_TX, CYBSP_DEBUG_UART_RX, CY_RETARGET_IO_BAUDRATE);

	/* retarget-io init failed. Stop program execution. */
	if (result != CY_RSLT_SUCCESS)
	{
		CY_ASSERT(0);
	}

    /* Initialize the user button. */
    result = cyhal_gpio_init(CYBSP_USER_BTN, CYHAL_GPIO_DIR_INPUT,
                             CYHAL_GPIO_DRIVE_PULLUP, CYBSP_BTN_OFF);
    CY_ASSERT(result == CY_RSLT_SUCCESS);

    /* Configure interrupt callback function, see cb_data for callback funtion. */
     cyhal_gpio_register_callback(CYBSP_USER_BTN, &cb_data);


    /* Enable interrupt. */
    cyhal_gpio_enable_event(CYBSP_USER_BTN, CYHAL_GPIO_IRQ_FALL,
                            GPIO_INTERRUPT_PRIORITY, true);

    /* Initialize the User LED. */
    result = cyhal_gpio_init(CYBSP_USER_LED, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, CYBSP_LED_STATE_OFF);
    CY_ASSERT(result == CY_RSLT_SUCCESS);

    /* Enable interrupts. */
    __enable_irq();

    /* Initialize DFU communication. */
    Cy_DFU_TransportStart();

    /* Keep the compiler happy. */
    (void)result;

    /*Print to debug port. */
    printf("Startup PSoC 6 DFU CE\r\n");

    for (;;)
    {
        status = Cy_DFU_Continue(&state, &dfu_params);

        ++count;
        if (state == CY_DFU_STATE_FINISHED)
        {
        	 /*Print to debug port. */
        	  printf("Startup PSoC 6 DFU CE- BLE DFU APP\r\n");

           /* Finished loading the application image.
            * Validate DFU application, if it is valid then switch to it.
            */

            status = manage_bootable_app(&dfu_params);
           if (status == CY_DFU_SUCCESS)
           {
               Cy_DFU_TransportStop();
               Cy_DFU_ExecuteApp(USER_APP_ID);
           }
           else if (status == CY_DFU_ERROR_VERIFY)
           {
               /* Restarts loading, an alternatives are to Halt MCU here
                * or switch to the other app if it is valid.
                * Error code may be handled here, i.e. print to debug UART.
                */
               status = restart_dfu(&state, &dfu_params );
           }
        }
        else if (state == CY_DFU_STATE_FAILED)
        {
           /* An error has happened during the loading process.
            * In this Code Example just restart loading process.
            * Restart DFU.
            */
            status = restart_dfu(&state, &dfu_params );
        }
        else if (state == CY_DFU_STATE_UPDATING)
        {
           timeout_seconds = (count >= counter_timeout_seconds(DFU_COMMAND_TIMEOUT_MS, DFU_SESSION_TIMEOUT_MS)) ? USER_APP_ID : 0u;

           /* if no command has been received during 5 seconds when the loading
            * has started then restart loading.
            */
           if (status == CY_DFU_SUCCESS)
           {
               count = 0u;
           }
           else if (status == CY_DFU_ERROR_TIMEOUT)
           {
               if (timeout_seconds != 0u)
               {
                   count = 0u;

                   /* Restart DFU. */
                   status = restart_dfu(&state, &dfu_params );
               }
           }
           else
           {
               count = 0u;

               /* Delay because Transport still may be sending error response to a host. */
               Cy_SysLib_Delay(DFU_SESSION_TIMEOUT_MS);

               /* Restart DFU. */
               status = restart_dfu(&state, &dfu_params );
           }
        }

        /* Blink once per seconds */
        if ( ( count % counter_timeout_seconds(LED_TOGGLE_INTERVAL_MS, DFU_SESSION_TIMEOUT_MS)) == 0u)
        {
            /* Invert the USER LED state */
            cyhal_gpio_toggle(CYBSP_USER_LED);
        }

        if((count >= counter_timeout_seconds(DFU_IDLE_TIMEOUT_MS, DFU_SESSION_TIMEOUT_MS) ) && (state == CY_DFU_STATE_NONE))
        {
            status = manage_bootable_app(&dfu_params);

            if (status == CY_DFU_SUCCESS)
            {
                Cy_DFU_TransportStop();
                Cy_DFU_ExecuteApp(USER_APP_ID);
            }

            /* In case, no valid user application, lets start fresh all over.
             * This is just for demonstration.
             * Final application can change it to either assert, reboot, enter low power mode etc,
             * based on usecase requirements.
             */
            count = 0 ;
        }

        /* Is user_button event detected ?. */
        if(is_user_event_detected == true)
        {
            is_user_event_detected = false;

            /* Boot to application image, if valid. */
            status = manage_bootable_app(&dfu_params);
            if (status == CY_DFU_SUCCESS)
            {
                /* Freeup the resources acquired. */
                cyhal_gpio_free(CYBSP_USER_BTN);

               /* Never returns from here. */
               Cy_DFU_ExecuteApp(USER_APP_ID);
            }
        }
    }
}

/*******************************************************************************
* Function Name: counter_timeout_seconds
********************************************************************************
* Returns number of counts that correspond to number of seconds passed as
* a parameter.
* E.g. comparing counter with 300 seconds is like this.
* ---
* uint32_t counter = 0u;
* for (;;)
* {
*     Cy_SysLib_Delay(UART_TIMEOUT);
*     ++count;
*     if (count >= counter_timeout_seconds(seconds: 300u, timeout: UART_TIMEOUT))
*     {
*         count = 0u;
*         DoSomething();
*     }
* }
* ---
*
* Both parameters are required to be compile time constants,
* so this function gets optimized out to single constant value.
*
* Parameters:
*  seconds    Number of seconds to pass. Must be less that 4_294_967 seconds.
*  timeout    Timeout for Cy_DFU_Continue() function, in milliseconds.
*             Must be greater than zero.
*             It is recommended to be a value that produces no reminder
*             for this function to be precise.
* Return:
*  See description.
*******************************************************************************/
static uint32_t counter_timeout_seconds(uint32_t seconds, uint32_t timeout)
{
    uint32_t count = 1 ;

    if(timeout != 0 )
    {
        count = ( (seconds) / timeout);
    }

    return count;
}

/*******************************************************************************
* Function Name: ManageBootableApp
********************************************************************************
* Performs validation of application, clears the existing rest reason if
* a valid image is found.
*
* Parameters:
*  dfuParams     input DFU parameters.
*
* Returns:
*  CY_DFU_SUCCESS if operation is successful and an error code in case of failure.
*******************************************************************************/
static cy_en_dfu_status_t manage_bootable_app(cy_stc_dfu_params_t *dfu_params)
{
    cy_en_dfu_status_t status = CY_DFU_SUCCESS;

    if(!dfu_params)
    {
        status = CY_DFU_ERROR_UNKNOWN;
    }

    /* Satisfy the compiler. */
    (void)dfu_params;

    if (status == CY_DFU_SUCCESS)
    {
        /* Validate the APP in SECONDARY slot. */
        status = Cy_DFU_ValidateApp(USER_APP_ID, NULL);
    }

    if (status == CY_DFU_SUCCESS)
    {
        /*
        * Clear reset reason because Cy_DFU_ExecuteApp() performs a
        * software reset. Without clearing, two reset reasons would be present.
        */
        do
        {
            Cy_SysLib_ClearResetReason();
        }while(Cy_SysLib_GetResetReason() != 0 );
    }
    return status;
}

/*******************************************************************************
* Function Name: restart_dfu
********************************************************************************
* This function re-initializes the DFU and resets the DFU transport.
*
* Parameters:
*  dfu_params     input DFU parameters.
*  state          input current state of the DFU
*
* Return:
*  Status of operation.
*******************************************************************************/
static cy_en_dfu_status_t restart_dfu(uint32_t *state, cy_stc_dfu_params_t *dfu_params )
{
    cy_en_dfu_status_t status = CY_DFU_SUCCESS;

    if(!state || !dfu_params)
    {
        status = CY_DFU_ERROR_UNKNOWN;
    }

    /* Restart DFU process. */
    if( status == CY_DFU_SUCCESS)
    {
        status = Cy_DFU_Init(state, dfu_params);
    }

    if( status == CY_DFU_SUCCESS)
    {
        Cy_DFU_TransportReset();
    }

    return status;
}

/*******************************************************************************
* Function Name: copy_row
********************************************************************************
* Copies data from a source address to a flash row with the address destination.
* If "src" data is the same as "dest" data then no copy is performed.
*
* Parameters:
*  dest     Destination address. Has to be an address of the start of flash row.
*  src      Source address.
*  rowSize  Size of the flash row.
*
* Returns:
*  CY_DFU_SUCCESS if operation is successful. Error code in a case of failure.
*******************************************************************************/
static cy_en_dfu_status_t copy_row(uint32_t dest, uint32_t src, uint32_t rowSize, cy_stc_dfu_params_t * params)
{
    cy_en_dfu_status_t status;

    /* Save params->dataBuffer value. */
    uint8_t *buffer = params->dataBuffer;

    /* Compare "dest" and "src" content. */
    params->dataBuffer = (uint8_t *)src;
    status = Cy_DFU_ReadData(dest, rowSize, CY_DFU_IOCTL_COMPARE, params);

    /* Restore params->dataBuffer. */
    params->dataBuffer = buffer;

    /* If "dest" differs from "src" then copy "src" to "dest". */
    if (status != CY_DFU_SUCCESS)
    {
        (void) memcpy((void *) params->dataBuffer, (const void*)src, rowSize);
        status = Cy_DFU_WriteData(dest, rowSize, CY_DFU_IOCTL_WRITE, params);
    }
    /* Restore params->dataBuffer. */
    params->dataBuffer = buffer;

    return (status);
}

/*******************************************************************************
* Function Name: handle_metadata
********************************************************************************
* The goal of this function is to make DFU SDK metadata (MD) valid.
* The following algorithm is used (in C-like pseudo code):
* ---
* if (isValid(MD) == true)
* {   if (MDC != MD)
*         MDC = MD;
* } else
* {   if(isValid(MDC) )
*         MD = MDC;
*     else
*         MD = INITIAL_VALUE;
* }
* ---
* Here MD is metadata flash row, MDC is flash row with metadata copy,
* INITIAL_VALUE is known initial value.
*
* In this code example MDC is placed in the next flash row after the MD, and
* INITIAL_VALUE is MD with only CRC, App0 start and size initialized,
* all the other fields are not touched.
*
* Parameters:
*  params   A pointer to a DFU SDK parameters structure.
*
* Returns:
* - CY_DFU_SUCCESS when finished normally.
* - Any other status code on error.
*******************************************************************************/
static cy_en_dfu_status_t handle_metadata(cy_stc_dfu_params_t *params)
{
    const uint32_t MD     = (uint32_t)(&__cy_boot_metadata_addr   ); /* MD address.  */
    const uint32_t mdSize = (uint32_t)(&__cy_boot_metadata_length ); /* MD size, assumed to be one flash row .*/
    const uint32_t MDC    = MD + mdSize;                             /* MDC address. */

    cy_en_dfu_status_t status = CY_DFU_SUCCESS;

    status = Cy_DFU_ValidateMetadata(MD, params);
    if (status == CY_DFU_SUCCESS)
    {
        /* Checks if MDC equals to DC, if no then copies MD to MDC. */
        status = copy_row(MDC, MD, mdSize, params);
    }
    else
    {
        status = Cy_DFU_ValidateMetadata(MDC, params);
        if (status == CY_DFU_SUCCESS)
        {
            /* Copy MDC to MD. */
            status = copy_row(MD, MDC, mdSize, params);
        }

        /* Possibly a clean boot or first boot. */
        if (status != CY_DFU_SUCCESS)
        {
            const uint32_t elfStartAddress = 0x10000000;
            const uint32_t elfAppSize      = 0x8000;

            /* Set MD to INITIAL_VALUE. */
            status = Cy_DFU_SetAppMetadata(0u, elfStartAddress, elfAppSize, params);
        }
    }
    return (status);
}

/*******************************************************************************
* Function Name: Cy_OnResetUser
********************************************************************************
*
*  This function is called at the start of Reset_Handler().
*  DFU requires it to call Cy_DFU_OnResetApp0() in app#0.
*
*******************************************************************************/
void Cy_OnResetUser(void)
{
    Cy_DFU_OnResetApp0();
}

/*******************************************************************************
* Function Name: user_button_event_handler
********************************************************************************
* Summary:
*   User Button event handler.
*
* Parameters:
*  void *handler_arg (unused)
*  cyhal_gpio_irq_event_t (unused)
*
*******************************************************************************/
static void user_button_event_handler(void *handler_arg, cyhal_gpio_event_t event)
{
    is_user_event_detected = true;
}


/*******************************************************************************
* Function Name: AppCallBack()
********************************************************************************
*
* Summary:
*   This is an event callback function to receive events from the BLE Component.
*   Used in Cy_DFU_TransportStart()
*
*  event - the event code
*  *eventParam - the event parameters
*
*******************************************************************************/
void AppCallBack(uint32 event, void* eventParam)
{
    cy_en_ble_api_result_t apiResult;

    static cy_stc_ble_gap_sec_key_info_t keyInfo =
    {
        .localKeysFlag = CY_BLE_GAP_SMP_INIT_ENC_KEY_DIST |
                         CY_BLE_GAP_SMP_INIT_IRK_KEY_DIST |
                         CY_BLE_GAP_SMP_INIT_CSRK_KEY_DIST,
        .exchangeKeysFlag = CY_BLE_GAP_SMP_INIT_ENC_KEY_DIST |
                            CY_BLE_GAP_SMP_INIT_IRK_KEY_DIST |
                            CY_BLE_GAP_SMP_INIT_CSRK_KEY_DIST |
                            CY_BLE_GAP_SMP_RESP_ENC_KEY_DIST |
                            CY_BLE_GAP_SMP_RESP_IRK_KEY_DIST |
                            CY_BLE_GAP_SMP_RESP_CSRK_KEY_DIST,
    };

    switch (event)
    {
    /**********************************************************
    *                       General Events
    ***********************************************************/

    /* This event received when BLE communication starts */
    case CY_BLE_EVT_STACK_ON:
        /* Enter into discoverable mode so that remote can search it. */
        apiResult = Cy_BLE_GAPP_StartAdvertisement(CY_BLE_ADVERTISING_FAST, 0u);
        if(apiResult != CY_BLE_SUCCESS)
        {
        }

        apiResult = Cy_BLE_GAP_GenerateKeys(&keyInfo);
        if(apiResult != CY_BLE_SUCCESS)
        {
            DBG_PRINTF("CyBle_GapGenerateKeys API Error: %d \r\n", apiResult);
        }
        break;

    /* This event indicates that some internal HW error has occurred. */
    case CY_BLE_EVT_HARDWARE_ERROR:
        DBG_PRINTF("CYBLE_EVT_HARDWARE_ERROR\r\n");
        break;
    /**********************************************************
    *                       GAP Events
    ***********************************************************/
    case CY_BLE_EVT_GAP_AUTH_REQ:
        DBG_PRINTF("CYBLE_EVT_AUTH_REQ: security=%x, bonding=%x, ekeySize=%x, err=%x \r\n",
            (*(cy_stc_ble_gap_auth_info_t *)eventParam).security,
            (*(cy_stc_ble_gap_auth_info_t *)eventParam).bonding,
            (*(cy_stc_ble_gap_auth_info_t*)eventParam).ekeySize,
            (*(cy_stc_ble_gap_auth_info_t *)eventParam).authErr    );
        if ( cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX].security
             == (CY_BLE_GAP_SEC_MODE_1 | CY_BLE_GAP_SEC_LEVEL_1)  )
        {
            cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX].authErr =
                CY_BLE_GAP_AUTH_ERROR_PAIRING_NOT_SUPPORTED;
        }

        cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX].bdHandle =
            ((cy_stc_ble_gap_auth_info_t *)eventParam)->bdHandle;

        apiResult = Cy_BLE_GAPP_AuthReqReply(&cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX]);
        if(apiResult != CY_BLE_SUCCESS)
        {
            Cy_BLE_GAP_RemoveOldestDeviceFromBondedList();
            apiResult = Cy_BLE_GAPP_AuthReqReply(&cy_ble_configPtr->authInfo[CY_BLE_SECURITY_CONFIGURATION_0_INDEX]);
            if(apiResult != CY_BLE_SUCCESS)
            {
               DBG_PRINTF("CyBle_GappAuthReqReply API Error: %d \r\n", apiResult);
            }
        }
        break;

    case CY_BLE_EVT_GAP_PASSKEY_ENTRY_REQUEST:
        DBG_PRINTF("CYBLE_EVT_PASSKEY_ENTRY_REQUEST press 'p' to enter passkey \r\n");
        break;

    case CY_BLE_EVT_GAP_PASSKEY_DISPLAY_REQUEST:
        DBG_PRINTF("CYBLE_EVT_PASSKEY_DISPLAY_REQUEST %6.6d \r\n", *(int *)eventParam);
        break;

    case CY_BLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT:
        DBG_PRINTF("CYBLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT \r\n");
        break;

    case CY_BLE_EVT_GAP_AUTH_COMPLETE:
        DBG_PRINTF("AUTH_COMPLETE \r\n");
        cyhal_gpio_write((cyhal_gpio_t)CYBSP_USER_LED2, CYBSP_LED_STATE_ON);
        break;

    case CY_BLE_EVT_GAP_AUTH_FAILED:
        DBG_PRINTF("CYBLE_EVT_AUTH_FAILED: %x \r\n", *(uint8 *)eventParam);
        break;

    case CY_BLE_EVT_GAP_DEVICE_CONNECTED:
        DBG_PRINTF("CYBLE_EVT_GAP_DEVICE_CONNECTED: %d \r\n", appConnHandle.bdHandle);

        if ( ((*(cy_stc_ble_gap_connected_param_t *)eventParam).connIntv
             < CYBLE_GAPP_CONNECTION_INTERVAL_MIN ) || (
             (*(cy_stc_ble_gap_connected_param_t *)eventParam).connIntv
             > CYBLE_GAPP_CONNECTION_INTERVAL_MAX ) )
        {
            cy_stc_ble_gap_conn_update_param_info_t connUpdateParam;
            /* If connection settings do not match expected ones - request parameter update */
            connUpdateParam.connIntvMin   = CYBLE_GAPP_CONNECTION_INTERVAL_MIN;
            connUpdateParam.connIntvMax   = CYBLE_GAPP_CONNECTION_INTERVAL_MAX;
            connUpdateParam.connLatency   = CYBLE_GAPP_CONNECTION_SLAVE_LATENCY;
            connUpdateParam.supervisionTO = CYBLE_GAPP_CONNECTION_TIME_OUT;
            connUpdateParam.bdHandle = appConnHandle.bdHandle;
            apiResult = Cy_BLE_L2CAP_LeConnectionParamUpdateRequest(&connUpdateParam);
            DBG_PRINTF("Cy_BLE_L2CAP_LeConnectionParamUpdateRequest API: 0x%2.2x \r\n", apiResult);
        }
        keyInfo.SecKeyParam.bdHandle = (*(cy_stc_ble_gap_connected_param_t *)eventParam).bdHandle;
        apiResult = Cy_BLE_GAP_SetSecurityKeys(&keyInfo);
        if(apiResult != CY_BLE_SUCCESS)
        {
            DBG_PRINTF("CyBle_GapSetSecurityKeys API Error: %d \r\n", apiResult);
        }
        break;

    case CY_BLE_EVT_L2CAP_CONN_PARAM_UPDATE_RSP:
        DBG_PRINTF("CY_BLE_EVT_L2CAP_CONN_PARAM_UPDATE_RSP, result = %d\r\n",
        (*(cy_stc_ble_l2cap_conn_update_rsp_param_t *)eventParam).result);
        break;

    case CY_BLE_EVT_GAP_KEYS_GEN_COMPLETE:
        DBG_PRINTF("CYBLE_EVT_GAP_KEYS_GEN_COMPLETE \r\n");
        keyInfo.SecKeyParam = (*(cy_stc_ble_gap_sec_key_param_t *)eventParam);
        Cy_BLE_GAP_SetIdAddress(&cy_ble_deviceAddress);
        break;

    case CY_BLE_EVT_GAP_DEVICE_DISCONNECTED:
        DBG_PRINTF("CYBLE_EVT_GAP_DEVICE_DISCONNECTED\r\n");
        cyhal_gpio_write((cyhal_gpio_t)CYBSP_USER_LED2, CYBSP_LED_STATE_OFF);
        /* Put the device into discoverable mode so that a remote can search it. */
        apiResult = Cy_BLE_GAPP_StartAdvertisement(CY_BLE_ADVERTISING_FAST, CY_BLE_PERIPHERAL_CONFIGURATION_0_INDEX);
        if(apiResult != CY_BLE_SUCCESS)
        {
            DBG_PRINTF("StartAdvertisement API Error: %d \r\n", apiResult);
        }
        break;

    case CY_BLE_EVT_GAP_ENCRYPT_CHANGE:
        DBG_PRINTF("CYBLE_EVT_GAP_ENCRYPT_CHANGE: %x \r\n", *(uint8 *)eventParam);
        break;

    case CY_BLE_EVT_GAP_CONNECTION_UPDATE_COMPLETE:
        DBG_PRINTF("CYBLE_EVT_CONNECTION_UPDATE_COMPLETE: %x \r\n", *(uint8 *)eventParam);
        cyhal_gpio_write((cyhal_gpio_t)CYBSP_USER_LED2, CYBSP_LED_STATE_ON);
        break;

    case CY_BLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
        if(Cy_BLE_GetAdvertisementState() == CY_BLE_ADV_STATE_STOPPED)
        {
            /* Fast and slow advertising period complete, go to low power
             * mode (Hibernate mode) and wait for an external
             * user event to wake up the device again */

            /* Stop DFU communication */
            Cy_DFU_TransportStop();
            Cy_SysPm_Hibernate();
        }
        break;

    /**********************************************************
    *                       GATT Events
    ***********************************************************/
    case CY_BLE_EVT_GATT_CONNECT_IND:
        appConnHandle = *(cy_stc_ble_conn_handle_t *)eventParam;
        DBG_PRINTF("CYBLE_EVT_GATT_CONNECT_IND: %d \r\n", appConnHandle.bdHandle);
        break;

    case CY_BLE_EVT_GATT_DISCONNECT_IND:
        DBG_PRINTF("CYBLE_EVT_GATT_DISCONNECT_IND: %d \r\n", ((cy_stc_ble_conn_handle_t *)eventParam)->bdHandle);
        break;

    case CY_BLE_EVT_GATTS_WRITE_CMD_REQ:
        DBG_PRINTF("CYBLE_EVT_GATTS_WRITE_CMD_REQ\r\n");
        break;

    default:
        break;
    }
}

/*******************************************************************************
* Function Name: ble_ias_callback
********************************************************************************
* Summary:
*  This is an event callback function to receive events from the BLE, which are
*  specific to Immediate Alert Service.
*
* Parameters:
*  uint32 event:      event from the BLE component
*  void* eventParams: parameters related to the event
*
*******************************************************************************/
void ble_ias_callback(uint32 event, void *eventParam)
{
    /* Alert Level Characteristic write event */
    if(event == CY_BLE_EVT_IASS_WRITE_CHAR_CMD)
    {
        /* Read the updated Alert Level value from the GATT database */
        Cy_BLE_IASS_GetCharacteristicValue(CY_BLE_IAS_ALERT_LEVEL,
                                           sizeof(alert_level), &alert_level);
        DBG_PRINTF("Alert Level: %u \r\n", (uint8)alert_level);
    }

    /* Remove warning for unused parameter */
    (void)eventParam;
}

/*******************************************************************************
* Function Name: ble_findme_status
********************************************************************************
* Summary:
*  This function updates the status of the LED
*  CY_BLE_NO
*
*******************************************************************************/
void ble_findme_status(void)
{
	static uint32 toggleCount = 0;

    /* Update CYBSP_USER_LED2 to indicate current alert level */
    switch(alert_level)
    {
        case CY_BLE_NO_ALERT:
        {
            cyhal_gpio_write((cyhal_gpio_t)CYBSP_USER_LED1, CYBSP_LED_STATE_OFF);
            break;
        }
        case CY_BLE_MILD_ALERT:
        {
        	toggleCount++;
          	if(toggleCount > 20)
           	{
                cyhal_gpio_toggle((cyhal_gpio_t)CYBSP_USER_LED1);
                toggleCount = 0;
          	}
            break;
        }
        case CY_BLE_HIGH_ALERT:
        {
            cyhal_gpio_write((cyhal_gpio_t)CYBSP_USER_LED1, CYBSP_LED_STATE_ON);
            break;
        }
        default:
        {
            break;
        }
    }
}

/* [] END OF FILE */
